package com.ruoyi.system.kernel;

import com.ruoyi.common.core.domain.R;
import java.util.List;
import java.time.ZonedDateTime;
import com.ruoyi.system.api.domain.SysFile;
import com.ruoyi.common.core.constant.Constants;
import com.ruoyi.common.core.domain.StrFile;
import com.ruoyi.system.utils.jsch.GeneratedSSLUtil;
import com.ruoyi.common.core.utils.file.FileUtils;
import com.ruoyi.system.domain.SnSslLog;
import java.util.Date;
import java.time.ZoneId;
import java.time.LocalDate;
import com.ruoyi.common.core.web.domain.AjaxResult;
import org.apache.commons.lang3.tuple.Triple;
import com.ruoyi.system.domain.SnServer;
import com.ruoyi.system.domain.dto.DeploySSLParams;
import com.ruoyi.system.api.RemoteFileService;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.system.mapper.SnSslLogMapper;
import org.springframework.stereotype.Service;

@Service
public class SslNginxConfigDealKernel
{
    @Autowired
    private SnSslLogMapper snSslLogMapper;
    @Autowired
    private RemoteFileService remoteFileService;
    
    public Triple<AjaxResult, DeploySSLParams, String> deal(final DeploySSLParams params, final SnServer server, final String domain, String nginxConfig) {
        if (params.getType() == 1) {
            final LocalDate fiveDaysAgo = LocalDate.now().plusDays(5L);
            final ZonedDateTime zdt = fiveDaysAgo.atStartOfDay(ZoneId.systemDefault());
            final Date date = Date.from(zdt.toInstant());
            final SnSslLog param = new SnSslLog();
            param.setDomain(domain);
            param.setExpireTime(date);
            final List<SnSslLog> logs = this.snSslLogMapper.selectSnSslLogList(param);
            if (!logs.isEmpty()) {
                params.setCacheSsl(Boolean.valueOf(true));
                final SnSslLog log = logs.get(0);
                final String pemUrl = log.getPem();
                final String pemName = FileUtils.getName(pemUrl);
                params.setSslPem(pemUrl);
                final String keyUrl = log.getKey();
                final String keyName = FileUtils.getName(keyUrl);
                params.setSslKey(keyUrl);
                nginxConfig = nginxConfig.replace("#{ssl-config}", "http2 on;\nssl_certificate /etc/ssl/" + domain + "/" + pemName + ";\nssl_certificate_key /etc/ssl/" + domain + "/" + keyName + ";\n");
            }
            else {
                final List<String> ssls = GeneratedSSLUtil.readNames(server, domain);
                final String pem = "/etc/ssl/" + domain + "/fullchain" + (ssls.size() + 1) + ".pem";
                final String key = "/etc/ssl/" + domain + "/privkey" + (ssls.size() + 1) + ".pem";
                params.setSslPemRoute(pem);
                params.setSslKeyRoute(key);
                nginxConfig = nginxConfig.replace("#{ssl-config}", "http2 on;\nssl_certificate " + pem + ";\nssl_certificate_key " + key + ";\n");
            }
        }
        else {
            if (params.getType() != 2) {
                return Triple.of(AjaxResult.error("暂不支持该证书类型"), params, nginxConfig);
            }
            StrFile strFile = new StrFile();
            strFile.setContent(params.getSslPem());
            strFile.setFileName("fullchain.pem");
            strFile.setContentType("text/plain");
            R<SysFile> r = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
            if (r.getCode() != Constants.SUCCESS) {
                return Triple.of(AjaxResult.error(r.getMsg()), params, nginxConfig);
            }
            final String pemUrl2 = ((SysFile)r.getData()).getUrl();
            final String pemName2 = ((SysFile)r.getData()).getName();
            params.setSslPem(pemUrl2);
            strFile = new StrFile();
            strFile.setContent(params.getSslKey());
            strFile.setFileName("privkey.pem");
            strFile.setContentType("text/plain");
            r = (R<SysFile>)this.remoteFileService.uploadStrToMinio(strFile);
            if (r.getCode() != Constants.SUCCESS) {
                return Triple.of(AjaxResult.error(r.getMsg()), params, nginxConfig);
            }
            final String keyUrl2 = ((SysFile)r.getData()).getUrl();
            final String keyName2 = ((SysFile)r.getData()).getName();
            params.setSslKey(keyUrl2);
            nginxConfig = nginxConfig.replace("#{ssl-config}", "http2 on;\nssl_certificate /etc/ssl/" + domain + "/" + pemName2 + ";\nssl_certificate_key /etc/ssl/" + domain + "/" + keyName2 + ";\n");
        }
        nginxConfig = nginxConfig.replace("#{server_name}", domain);
        nginxConfig = nginxConfig.replace("#{open_ssl}", "");
        return Triple.of(AjaxResult.success(), params, nginxConfig);
    }
}
